@@ -9,8 +9,8 @@ class Caching:
     def get(self, secret):
         return self._cache.get(self._hash(secret))
 
-    def set(self, secret):
-        return self._cache.set(self._hash(secret), True)
+    def set(self, secret, value):
+        return self._cache.set(self._hash(secret), value)
 
     def _hash(self, secret):
         hash_object = hashlib.md5(secret.encode())
